% testing the different functions
clear; close all;

N_time = 10;                % number of evaluations when doing the timing   

param.N = 1000;             % number of discretization steps
param.x0 = 2;               % initial condition
param.T  = 5;               % terminal time
param.q  = 20;              % terminal weight
h = param.T/param.N;

Utst = rand(param.N,1);     % random control trajectory

% finite differences on nonlinear part
time_FD = 0;
for i = 1:N_time
    tic
    [F1, J1] = finite_difference(@Phi, Utst, param);
    time_FD = time_FD + toc;
end
time_FD = time_FD / N_time;

% imaginary trick
time_IT = 0;
for i = 1:N_time
    tic
    [F2, J2] = i_trick(@Phi, Utst, param);
    time_IT = time_IT + toc;
end
time_IT = time_IT / N_time;

% forward AD
time_FAD = 0;
for i = 1:N_time
    tic
    [F3, J3] = Phi_FAD(Utst, param);
    time_FAD = time_FAD + toc;
end
time_FAD = time_FAD / N_time;


% backward AD
time_BAD = 0;
for i = 1:N_time
    tic
    [F4, J4] = Phi_BAD(Utst, param);
    time_BAD = time_BAD + toc;
end
time_BAD = time_BAD / N_time;

% running this script adds the casadi function Phi_casadi(U) to workspace
casadi_script_sol

% evaluate the casadi function
time_CA = 0;
for i = 1:N_time
    tic
    [F5, J5] = Phi_casadi(Utst);
    time_CA = time_CA + toc;
end
time_CA = time_CA / N_time;


% check results
disp('Error between imaginary trick and finite differences:')
disp(max(max(abs(J2-J1))))

disp('Error between imaginary trick and forward AD:')
disp(max(max(abs(J2-J3))))

disp('Error between imaginary trick and backward AD:')
disp(max(max(abs(J2-J4))))

disp('Error between imaginary trick and casadi:')
disp(' ')
disp(max(max(abs(J2-J5))))


disp(['mean time FD: ', num2str(time_FD)])
disp(['mean time IT: ', num2str(time_IT)])
disp(['mean time FAD: ', num2str(time_FAD)])
disp(['mean time BAD: ', num2str(time_BAD)])
disp(['mean time CA: ', num2str(time_CA)])

figure
bar([time_FD time_IT time_FAD time_BAD time_CA])
xticks(1:5);
xticklabels({'FD','IT','FAD','BAD','CasADi'})
