import numpy as np


def Phi_FAD(u, param):
    N = np.size(u)
    x0 = param['x0']
    h = param['T'] / N
    q = param['q']

    # forward sweep
    x = np.zeros(N+1)
    x[0] = x0
    for i in range(N):
        x[i+1] = x[i] + h*((1 - x[i])*x[i] + u[i])

    f = q * x[N]**2

    # obtain Jacobian
    P = np.eye(N)  # for seeds
    J = np.zeros((1,N))

    # forward AD
    for i in range(N):
        p = P[:,i]     # seed vector
        udot = p       # seed input variable

        # obtain all dot quantities
        xdot = np.zeros(N+1)
        for k in range(N):
            df_dx = 1 + h*(1 - 2*x[k])
            df_du = h
            xdot[k+1] =  df_dx*xdot[k] + df_du*udot[k]

        # the dot quantity of output f is our jacobian entry
        J[:,i] = 2*q*x[N]*xdot[N]

    return (f, J)
